/* Copyright (c) 1989 by Carrick Sean Casey. All rights reserved. */
/* For copying and distribution information, see the file "copyright.h." */

/* locally processed commands */

#include "../copyright.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/dir.h>
#include "../config.h"
#include "../forumnet.h"
#include "externs.h"

static char logfile[MAXNAMLEN+1];	/* filename for session log */

/* mail a bug report to sean */
/* NOTE the address should be kept as a define in config.h */

cmd_bug(cmd, args)
char *cmd, *args;
{
	char cmdbuf[132];
	char *mailer, *getenv();

	puts("\n[sending mail to sean@ms.uky.edu]");
	restoreterm();
	if ((mailer = getenv("MAILER")) != NULL)
		sprintf(cmdbuf, "%s sean@ms.uky.edu", mailer);
	else
		sprintf(cmdbuf, "mail sean@ms.uky.edu");
	system(cmdbuf);
	puts("\n[returning to forumnet]");
	setterm();
}

/* use a pager on the help file */

cmd_help(cmd, args)
char *cmd, *args;
{
	char cmdbuf[132];
	char *pager, *getenv();

	restoreterm();
	if ((pager = getenv("PAGER")) != NULL)
		sprintf(cmdbuf, "%s %s/forum.help", pager, LIBDIR);
	else
		sprintf(cmdbuf, "%s %s %s/forum.help",
		  DEFAULT_PAGER, PAGEROPTS, LIBDIR);
	system(cmdbuf);
	puts("\n[returning to forumnet]");
	setterm();
}

/* quit */
/* when forum decides to exit it usually end up here */

cmd_quit(cmd, args)
char *cmd, *args;
{
	restoreterm();		/* retore term modes */
	closecon();		/* close the connection to the server */
	if (logisopen())
		cmd_nolog(0,0);	/* close the session log */
	exit(0);
}

/* list available commands */

cmd_list(cmd, args)
char *cmd, *args;
{
	/* request server to tell us about remote commands */
	sendcmd("c", "");
}

/* send a private message to another user */
/* this is first handled here, and then sent to the server */
/* we remember some info about who we sent to */

cmd_message(cmd, args)
char *cmd, *args;
{
	int ret;
	char *p, *findspace();

	/* see if there's a second arg */
	if ((p = findspace(args)) == 0) {
		if ((ret = getline(mbuf2, ">> ", 0)) < 0)
			cmd_quit();
		else if (ret == 0 || *mbuf2 == '\n') {
			putline("[message not sent]");
			return(0);
		} else {
			strcpy(mbuf, ">> ");
			strcat(mbuf, mbuf2);
			lputline(mbuf);
			sprintf(mbuf, "%s %s", args, mbuf2);
			histput(args);
			sendcmd(cmd, mbuf);
			return(0);
		}
			
	} else {
		histput(args);
		/* findspace replaces first space with a null -- reset it */
		*--p = ' ';
		sendcmd(cmd, args);
	}
}

/* print the time of day */

cmd_time(cmd, args)
char *cmd, *args;
{
	char *qtime(), *btime();

	if (m_watchtime) {
		sprintf(mbuf, "The time is %s.\n", btime());
		putline(mbuf);
	} else
		putline(qtime());
}

/* start a session log */

cmd_log(cmd, args)
char *cmd, *args;
{
	char *aptr, *ampm();
	time_t time();
	struct tm *t, *localtime();
	time_t clock;

	/* see if we're already logging */
	if (logisopen()) {
		sprintf(mbuf, "(log) Already session logging into \"%s\".",
		  logfile);
		puterr(mbuf);
		return(0);
	}

	if (*args != '\0')
		/* NOTE here's where we'd parse a tilde */
		strcpy(logfile, args);
	else
		strcpy(logfile, "forum.log");
	
	/* go open the log */
	if (!opensesslog(logfile))
		return(0);

	/* get the time so we can timestamp it */
	time(&clock);
	t = localtime(&clock);
	aptr = ampm(clock);
	if (*aptr == ' ')
		aptr++;
	
	sprintf(mbuf, "Session log \"%s\" started at %d/%02d/%-2d %s.",
	  logfile, t->tm_mon+1, t->tm_mday, t->tm_year, aptr);
	putline(mbuf);
}

/* close a session log */

cmd_nolog(cmd, args)
char *cmd, *args;
{
	char *aptr, *ampm();
	time_t time();
	struct tm *t, *localtime();
	time_t clock;

	/* make sure we're actually logging */
	if (!logisopen()) {
		sprintf(mbuf, "(nolog) Session logging is not active.",
		  logfile);
		puterr(mbuf);
		return(0);
	}

	/* get the time so we can timestamp it (deja vu?) */
	time(&clock);
	t = localtime(&clock);
	aptr = ampm(clock);
	if (*aptr == ' ')
		aptr++;
	
	sprintf(mbuf, "Session log \"%s\" closed at %d/%02d/%-2d %s.",
	  logfile, t->tm_mon+1, t->tm_mday, t->tm_year, aptr);
	putline(mbuf);

	/* close the log file */
	closesesslog();
}

/* turn on beeping */

cmd_alert(cmd, args)
char *cmd, *args;
{
	/* make sure we're actually logging */
	if (beeping) {
		puterr("Alert is already enabled (BEEP!).");
		return(0);
	}
	beeping = 1;
	putline("Alert is now enabled (BEEP!).");
}


/* turn off beeping */

cmd_noalert(cmd, args)
char *cmd, *args;
{

	/* make sure we're actually logging */
	if (!beeping) {
		puterr("Alert is already off.");
		return(0);
	}
	beeping = 0;
	putline("Alert is now disabled (silence).");
}

/* execute a command under a subshell */

cmd_shell(cmd, args)
char *cmd, *args;
{
	char *shell, *getenv();

	restoreterm();

	if (*args != '\0') {
		/* user gave command to run */
		printf("[%s]\n", args);
		system(args);
	} else {
		/* user didn't specify command - run a subshell */
		/* get shell to use */
		if ((shell = getenv("SHELL")) == NULL)
			shell = "/bin/sh";
		printf("[%s]\n", shell);
		system(shell);
	}
	puts("\n[returning to forumnet]");
	setterm();
}

cmd_ccommand(cmd, args)
char *cmd, *args;
{
	/* find the first non-whitespace char */
	while ((*args == ' ' || *args == '\t') && *args != '\0')
		args++;

	if (*args == '\0') {
		/* just display it */
		sprintf(mbuf, "The current command prefix is \"%c\".", cmdchar);
		putline(mbuf);
		return(0);
	}
	
	sprintf(mbuf, "Command prefix changed from \"%c\" to \"%c\".",
		cmdchar, *args);
	putline(mbuf);
	cmdchar = *args;
}
